package de.fhdw.gaming.core.domain.util;

import java.util.Optional;
import java.util.Set;

import de.fhdw.gaming.core.domain.Game;
import de.fhdw.gaming.core.domain.Move;
import de.fhdw.gaming.core.domain.Observer;
import de.fhdw.gaming.core.domain.Player;
import de.fhdw.gaming.core.domain.State;

/**
 * Creates a list of events.
 */
@SuppressWarnings("PMD.AvoidDuplicateLiterals")
public final class EventListCreator {

    /**
     * The resulting string.
     */
    private final StringBuilder result = new StringBuilder();

    /**
     * Returns the result.
     */
    public String getResult() {
        return this.result.toString();
    }

    /**
     * See {@link Observer#started(Game, State)}.
     *
     * @param state See {@link Observer#started(Game, State)}.
     * @return {@code this}
     */
    public EventListCreator started(final State<?, ?> state) {
        this.result.append("started").append("\n");
        return this;
    }

    /**
     * See {@link Observer#nextPlayersComputed(Game, State, Set)}.
     *
     * @param state   See {@link Observer#nextPlayersComputed(Game, State, Set)}.
     * @param players See {@link Observer#nextPlayersComputed(Game, State, Set)}.
     * @return {@code this}
     */
    public EventListCreator nextPlayersComputed(final State<?, ?> state, final Set<? extends Player<?>> players) {
        this.result.append("nextPlayersComputed: ").append("players=").append(players).append("\n");
        return this;
    }

    /**
     * See {@link Observer#illegalPlayerRejected(Game, State, Player)}.
     *
     * @param state  See {@link Observer#illegalPlayerRejected(Game, State, Player)}.
     * @param player See {@link Observer#illegalPlayerRejected(Game, State, Player)}.
     * @return {@code this}
     */
    public EventListCreator illegalPlayerRejected(final State<?, ?> state, final Player<?> player) {
        this.result.append("illegalPlayerRejected: ").append("player=").append(player).append("\n");
        return this;
    }

    /**
     * See {@link Observer#legalMoveApplied(Game, State, Player, Move)}.
     *
     * @param state  See {@link Observer#legalMoveApplied(Game, State, Player, Move)}.
     * @param player See {@link Observer#legalMoveApplied(Game, State, Player, Move)}.
     * @param move   See {@link Observer#legalMoveApplied(Game, State, Player, Move)}.
     * @return {@code this}
     */
    public EventListCreator legalMoveApplied(final State<?, ?> state, final Player<?> player, final Move<?, ?> move) {
        this.result.append("legalMoveApplied: ").append("player=").append(player).append(", move=").append(move)
                .append("\n");
        return this;
    }

    /**
     * See {@link Observer#illegalMoveRejected(Game, State, Player, Optional, String)}.
     *
     * @param state  See {@link Observer#illegalMoveRejected(Game, State, Player, Optional, String)}.
     * @param player See {@link Observer#illegalMoveRejected(Game, State, Player, Optional, String)}.
     * @param move   See {@link Observer#illegalMoveRejected(Game, State, Player, Optional, String)}.
     * @param reason See {@link Observer#illegalMoveRejected(Game, State, Player, Optional, String)}.
     * @return {@code this}
     */
    public EventListCreator illegalMoveRejected(final State<?, ?> state, final Player<?> player,
            final Optional<Move<?, ?>> move, final String reason) {
        this.result.append("illegalMoveRejected: ").append("player=").append(player).append(", move=").append(move)
                .append(", reason=").append(reason).append("\n");
        return this;
    }

    /**
     * See {@link Observer#overdueMoveRejected(Game, State, Player, Optional)}.
     *
     * @param state      See {@link Observer#overdueMoveRejected(Game, State, Player, Optional)}.
     * @param player     See {@link Observer#overdueMoveRejected(Game, State, Player, Optional)}.
     * @param chosenMove See {@link Observer#overdueMoveRejected(Game, State, Player, Optional)}.
     * @return {@code this}
     */
    public EventListCreator overdueMoveRejected(final State<?, ?> state, final Player<?> player,
            final Optional<Move<?, ?>> chosenMove) {
        this.result.append("overdueMoveRejected: ").append("player=").append(player).append(", chosenMove=")
                .append(chosenMove).append("\n");
        return this;
    }

    /**
     * See {@link Observer#playerResigned(Game, State, Player)}.
     *
     * @param state  See {@link Observer#playerResigned(Game, State, Player)}.
     * @param player See {@link Observer#playerResigned(Game, State, Player)}.
     * @return {@code this}
     */
    public EventListCreator playerResigned(final State<?, ?> state, final Player<?> player) {
        this.result.append("playerResigned: ").append("player=").append(player).append("\n");
        return this;
    }

    /**
     * See {@link Observer#playerOvertaken(Game, State, Player, Player)}.
     *
     * @param state            See {@link Observer#playerOvertaken(Game, State, Player, Player)}.
     * @param overtakenPlayer  See {@link Observer#playerOvertaken(Game, State, Player, Player)}.
     * @param overtakingPlayer See {@link Observer#playerOvertaken(Game, State, Player, Player)}.
     * @return {@code this}
     */
    public EventListCreator playerOvertaken(final State<?, ?> state, final Player<?> overtakenPlayer,
            final Player<?> overtakingPlayer) {
        this.result.append("playerOvertaken: ").append("overtakenPlayer=").append(overtakenPlayer)
                .append(", overtakingPlayer=").append(overtakingPlayer).append("\n");
        return this;
    }

    /**
     * See {@link Observer#finished(Game, State)}.
     *
     * @param state See {@link Observer#finished(Game, State)}.
     * @return {@code this}
     */
    public EventListCreator finished(final State<?, ?> state) {
        this.result.append("finished").append("\n");
        return this;
    }
}
