/*
 * Copyright © 2020 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of gaming-core.
 *
 * Gaming-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Gaming-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * gaming-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.core.ui.type;

import java.util.Optional;

import de.fhdw.gaming.core.ui.InputProviderException;

/**
 * Represents the type of a boolean field.
 */
public final class BooleanFieldType implements FieldType<Boolean> {

    /**
     * The default value.
     */
    private final Optional<Boolean> defaultValue;

    /**
     * Creates a {@link BooleanFieldType}.
     *
     * @param defaultValue The default value.
     */
    public BooleanFieldType(final Optional<Boolean> defaultValue) {
        this.defaultValue = defaultValue;
    }

    @Override
    public void accept(final FieldTypeVisitor visitor) throws FieldTypeException {
        visitor.handle(this);
    }

    @Override
    public Optional<Boolean> getDefaultValue() {
        return this.defaultValue;
    }

    @Override
    public Optional<String> getInfo() {
        return Optional.of("The value must be either true/yes or false/no.");
    }

    @Override
    public Boolean postInput(final String id, final String value) throws InputProviderException {
        if (value.equalsIgnoreCase("true") || value.equalsIgnoreCase("yes")) {
            return Boolean.TRUE;
        } else if (value.equalsIgnoreCase("false") || value.equalsIgnoreCase("no")) {
            return Boolean.FALSE;
        } else {
            throw new InputProviderException(this.getInfo().orElseThrow());
        }
    }
}
