/*
 * Copyright © 2020 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of gaming-core.
 *
 * Gaming-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Gaming-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * gaming-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.core.ui.type;

import java.util.Optional;

/**
 * Represents the type of a string field.
 */
public final class StringFieldType implements FieldType<String> {

    /**
     * The default value.
     */
    private final Optional<String> defaultValue;

    /**
     * Creates a {@link StringFieldType}.
     *
     * @param defaultValue The default value.
     */
    public StringFieldType(final Optional<String> defaultValue) {
        this.defaultValue = defaultValue;
    }

    @Override
    public void accept(final FieldTypeVisitor visitor) throws FieldTypeException {
        visitor.handle(this);
    }

    @Override
    public Optional<String> getDefaultValue() {
        return this.defaultValue;
    }

    @Override
    public Optional<String> getInfo() {
        return Optional.empty();
    }

    @Override
    public String postInput(final String id, final String value) {
        return value;
    }
}
