package de.fhdw.gaming.core.ui.type.validator;

import de.fhdw.gaming.core.ui.InputProviderException;

/**
 * Validates against a maximum value.
 *
 * @param <T> The type of value.
 */
public final class MaxValueValidator<T extends Comparable<T>> implements Validator<T> {

    /**
     * The maximum value.
     */
    private final T maxValue;

    /**
     * Creates a {@link MaxValueValidator}.
     *
     * @param maxValue The maximum value.
     */
    public MaxValueValidator(final T maxValue) {
        this.maxValue = maxValue;
    }

    @Override
    public T validate(final T value) throws InputProviderException {
        if (value.compareTo(this.maxValue) > 0) {
            throw new InputProviderException(this.getInfo());
        } else {
            return value;
        }
    }

    @Override
    public String getInfo() {
        return String.format("The value must be at most %s.", this.maxValue);
    }
}
